<?php


namespace Zeedhi\DataMining\Common\Service\PeriodMathExpression;


use Moment\Moment;

class Period
{

    /**
     * @param string $expression
     * @return string
     * @throws \Exception
     */
    public static function getDate($expression = ''){

        $date = self::decode($expression);
        return $date->format('d/m/Y');
    }



    /**
     * @param $expression
     * @return array
     */
    protected static function getArrayExpression($expression)
    {
        $length = strlen($expression);
        $arrayExpression = array();
        $singleExpression = '';
        for ($i = 0; $i < $length; $i++) {
            $char = $expression[$i];
            if (in_array($char, Strategy\UnitFactory::OPERATIONS)) {
                $arrayExpression[] = $singleExpression;
                $singleExpression = '';
            }
            $singleExpression .= $char;

        }
        $arrayExpression[] = $singleExpression;
        return $arrayExpression;
    }

    public static function convertDateTimeToMoment(\DateTime $dateTime) {
        $moment = new Moment($dateTime->format('Y-m-d'));
        return $moment;
    }

    public static function getDateMoment($expression = ''){
        $dateTime = self::decode($expression);
        return self::convertDateTimeToMoment($dateTime);
    }

    /**
     * @param $expression
     * @return \DateTime
     * @throws \Exception
     */
    protected static function decode($expression): \DateTime
    {
        $date = new \DateTime();
        $arrayExpression = self::getArrayExpression($expression);
        foreach ($arrayExpression as $dateExpression) {
            if ($dateExpression) {
                $operator = $dateExpression[0];
                $unit = $dateExpression[strlen($dateExpression) - 1];
                $quantity = preg_replace("/[^0-9]/", "", $dateExpression);
                if (!$quantity && $quantity != '0') {
                    throw New \Exception('Quantity not found');
                }
                $strategy = Strategy\UnitFactory::factory($unit);
                $date = $strategy->modify($date, $operator, $quantity);
            }
        }
        return $date;
    }
}